@@ -12,18 +12,25 @@
 #
 ##############################################################################
 
-# At the moment, no ZODB release include the following patches.
-# Later, this must be replaced by some detection mechanism.
 if 1:
-
+    from hashlib import md5
     from ZODB.Connection import Connection
 
+    def _check(f, *args):
+        h = md5(f.func_code.co_code).hexdigest()
+        assert h in args, h
+
     # Allow serial to be returned as late as tpc_finish
     #
     # This makes possible for storage to allocate serial inside tpc_finish,
     # removing the requirement to serialise second commit phase (tpc_vote
     # to tpc_finish/tpc_abort).
 
+    _check(Connection.tpc_finish,
+        'f50ed2e5a74f584fa1ecaf1cd70a6b15', # Python 2.6
+        'ab9b1b8d82c40e5fffa84f7bc4ea3a8b', # Python 2.7
+        )
+
     def tpc_finish(self, transaction):
         """Indicate confirmation that the transaction is done."""
 
@@ -39,6 +46,7 @@ if 1:
 #       to be able to read any updated data until we've had a chance
 #       to send an invalidation message to all of the other
 #       connections!
+        # <patch>
         serial = self._storage.tpc_finish(transaction, callback)
         if serial is not None:
             assert isinstance(serial, str), repr(serial)
@@ -49,47 +57,11 @@ if 1:
                     if obj is not None and obj._p_changed is not None:
                         obj._p_changed = 0
                         obj._p_serial = serial
+        # </patch>
         self._tpc_cleanup()
 
     Connection.tpc_finish = tpc_finish
 
-    ###
-
-    try:
-        if Connection._nexedi_fix != 5:
-            raise Exception("A different ZODB fix is already applied")
-    except AttributeError:
-        Connection._nexedi_fix = 5
-
-        # Whenever an connection is opened (and there's usually an existing one
-        # in DB pool that can be reused) whereas the transaction is already
-        # started, we must make sure that proper storage setup is done by
-        # calling Connection.newTransaction.
-        # For example, there's no open transaction when a ZPublisher/Publish
-        # transaction begins.
-
-        def open(self, *args, **kw):
-            def _flush_invalidations():
-                acquire = self._db._a
-                try:
-                    self._db._r() # this is a RLock
-                except RuntimeError:
-                    acquire = lambda: None
-                try:
-                    del self._flush_invalidations
-                    self.newTransaction()
-                finally:
-                    acquire()
-                    self._flush_invalidations = _flush_invalidations
-            self._flush_invalidations = _flush_invalidations
-            try:
-                Connection_open(self, *args, **kw)
-            finally:
-                del self._flush_invalidations
-
-        Connection_open = Connection.open
-        Connection.open = open
-
     # IStorage implementations usually need to provide a "network barrier",
     # at least for NEO & ZEO, to make sure we have an up-to-date view of
     # the storage. It's unclear whether sync() is a good place to do this
@@ -99,11 +71,13 @@ if 1:
     # so this should not be done in afterCompletion(), and anyway, we don't
     # know any legitimate use of DB access outside a transaction.
 
+    _check(Connection.afterCompletion,
+        '70dfc3df8a455d9f663dec619c607eb5', # Python 2.6
+        'cd3a080b80fd957190ff3bb867149448', # Python 2.7
+        )
+
     def afterCompletion(self, *ignored):
-        try:
-            self._readCurrent.clear()
-        except AttributeError: # BBB: ZODB < 3.10
-            pass
+        self._readCurrent.clear()
         # PATCH: do not call sync()
         self._flush_invalidations()
     Connection.afterCompletion = afterCompletion
