@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2006-2012  Nexedi SA
+# Copyright (C) 2006-2014  Nexedi SA
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -15,11 +15,12 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 from cPickle import dumps, loads
-from zlib import compress as real_compress, decompress
+from zlib import compress, decompress
 from neo.lib.locking import Empty
 from random import shuffle
 import heapq
 import time
+from functools import partial
 
 from ZODB.POSException import UndoError, StorageTransactionError, ConflictError
 from ZODB.POSException import ReadConflictError
@@ -30,9 +31,9 @@ from neo.lib import logging
 from neo.lib.protocol import NodeTypes, Packets, \
     INVALID_PARTITION, ZERO_HASH, ZERO_TID
 from neo.lib.event import EventManager
-from neo.lib.util import makeChecksum as real_makeChecksum, dump
+from neo.lib.util import makeChecksum, dump
 from neo.lib.locking import Lock
-from neo.lib.connection import MTClientConnection, OnTimeout, ConnectionClosed
+from neo.lib.connection import MTClientConnection, ConnectionClosed
 from neo.lib.node import NodeManager
 from neo.lib.connector import getConnectorHandler
 from .exception import NEOStorageError, NEOStorageCreationUndoneError
@@ -44,25 +45,9 @@ from .iterator import Iterator
 from .cache import ClientCache
 from .pool import ConnectionPool
 from neo.lib.util import p64, u64, parseMasterList
-from neo.lib.profiling import profiler_decorator, PROFILING_ENABLED
 from neo.lib.debug import register as registerLiveDebugger
 from .container import ThreadContainer, TransactionContainer
 
-if PROFILING_ENABLED:
-    # Those functions require a "real" python function wrapper before they can
-    # be decorated.
-    @profiler_decorator
-    def compress(data):
-        return real_compress(data)
-
-    @profiler_decorator
-    def makeChecksum(data):
-        return real_makeChecksum(data)
-else:
-    # If profiling is disabled, directly use original functions.
-    compress = real_compress
-    makeChecksum = real_makeChecksum
-
 CHECKED_SERIAL = master.CHECKED_SERIAL
 
 
@@ -83,7 +68,6 @@ class Application(object):
         self.dispatcher = Dispatcher(self.poll_thread)
         self.nm = NodeManager(dynamic_master_list)
         self.cp = ConnectionPool(self)
-        self.pt = None
         self.master_conn = None
         self.primary_master_node = None
         self.trying_master_node = None
@@ -96,7 +80,7 @@ class Application(object):
         self.uuid = None
         self._cache = ClientCache()
         self._loading_oid = None
-        self.new_oid_list = []
+        self.new_oid_list = ()
         self.last_oid = '\0' * 8
         self.last_tid = None
         self.storage_event_handler = storage.StorageEventHandler(self)
@@ -122,11 +106,9 @@ class Application(object):
         # _cache_lock is used for the client cache
         self._cache_lock_acquire = lock.acquire
         self._cache_lock_release = lock.release
-        lock = Lock()
         # _connecting_to_master_node is used to prevent simultaneous master
         # node connection attemps
-        self._connecting_to_master_node_acquire = lock.acquire
-        self._connecting_to_master_node_release = lock.release
+        self._connecting_to_master_node = Lock()
         # _nm ensure exclusive access to the node manager
         lock = Lock()
         self._nm_acquire = lock.acquire
@@ -134,14 +116,21 @@ class Application(object):
         self.compress = compress
         registerLiveDebugger(on_log=self.log)
 
+    def __getattr__(self, attr):
+        if attr == 'pt':
+            self._getMasterConnection()
+        return self.__getattribute__(attr)
+
+    @property
+    def txn_contexts(self):
+        # do not iter lazily to avoid race condition
+        return self._txn_container.values
+
     def getHandlerData(self):
-        return self._thread_container.get()['answer']
+        return self._thread_container.answer
 
     def setHandlerData(self, data):
-        self._thread_container.get()['answer'] = data
-
-    def _getThreadQueue(self):
-        return self._thread_container.get()['queue']
+        self._thread_container.answer = data
 
     def log(self):
         self.em.log()
@@ -149,7 +138,6 @@ class Application(object):
         if self.pt is not None:
             self.pt.log()
 
-    @profiler_decorator
     def _handlePacket(self, conn, packet, kw={}, handler=None):
         """
           conn
@@ -179,7 +167,6 @@ class Application(object):
         finally:
             conn.unlock()
 
-    @profiler_decorator
     def _waitAnyMessage(self, queue, block=True):
         """
           Handle all pending packets.
@@ -217,10 +204,9 @@ class Application(object):
             # Don't leave access to thread context, even if a raise happens.
             self.setHandlerData(None)
 
-    @profiler_decorator
     def _ask(self, conn, packet, handler=None, **kw):
         self.setHandlerData(None)
-        queue = self._getThreadQueue()
+        queue = self._thread_container.queue
         msg_id = conn.ask(packet, queue=queue, **kw)
         get = queue.get
         _handlePacket = self._handlePacket
@@ -241,58 +227,47 @@ class Application(object):
                 _handlePacket(qconn, qpacket, kw)
         return self.getHandlerData()
 
-    @profiler_decorator
     def _askStorage(self, conn, packet, **kw):
         """ Send a request to a storage node and process its answer """
         return self._ask(conn, packet, handler=self.storage_handler, **kw)
 
-    @profiler_decorator
     def _askPrimary(self, packet, **kw):
         """ Send a request to the primary master and process its answer """
         return self._ask(self._getMasterConnection(), packet,
             handler=self.primary_handler, **kw)
 
-    @profiler_decorator
     def _getMasterConnection(self):
         """ Connect to the primary master node on demand """
-        # acquire the lock to allow only one thread to connect to the primary
+        # For performance reasons, get 'master_conn' without locking.
         result = self.master_conn
         if result is None:
-            self._connecting_to_master_node_acquire()
-            try:
-                self.new_oid_list = []
-                result = self._connectToPrimaryNode()
-            finally:
-                self._connecting_to_master_node_release()
+            # If not connected, 'master_conn' must be tested again while we have
+            # the lock, to avoid concurrent threads reconnecting.
+            with self._connecting_to_master_node:
+                result = self.master_conn
+                if result is None:
+                    self.new_oid_list = ()
+                    result = self.master_conn = self._connectToPrimaryNode()
         return result
 
-    def getPartitionTable(self):
-        """ Return the partition table manager, reconnect the PMN if needed """
-        # this ensure the master connection is established and the partition
-        # table is up to date.
-        self._getMasterConnection()
-        return self.pt
-
-    @profiler_decorator
     def _connectToPrimaryNode(self):
         """
             Lookup for the current primary master node
         """
         logging.debug('connecting to primary master...')
-        ready = False
-        nm = self.nm
-        while not ready:
+        index = 0
+        ask = self._ask
+        handler = self.primary_bootstrap_handler
+        while 1:
             # Get network connection to primary master
-            index = 0
-            connected = False
-            while not connected:
+            while 1:
                 if self.primary_master_node is not None:
                     # If I know a primary master node, pinpoint it.
                     self.trying_master_node = self.primary_master_node
                     self.primary_master_node = None
                 else:
                     # Otherwise, check one by one.
-                    master_list = nm.getMasterList()
+                    master_list = self.nm.getMasterList()
                     try:
                         self.trying_master_node = master_list[index]
                     except IndexError:
@@ -313,58 +288,52 @@ class Application(object):
                                   self.trying_master_node)
                     continue
                 try:
-                    self._ask(conn, Packets.RequestIdentification(
+                    ask(conn, Packets.RequestIdentification(
                             NodeTypes.CLIENT, self.uuid, None, self.name),
-                        handler=self.primary_bootstrap_handler)
+                        handler=handler)
                 except ConnectionClosed:
                     continue
                 # If we reached the primary master node, mark as connected
-                connected = self.primary_master_node is not None and \
-                        self.primary_master_node is self.trying_master_node
+                if self.primary_master_node is not None and \
+                   self.primary_master_node is self.trying_master_node:
+                    break
             logging.info('Connected to %s', self.primary_master_node)
             try:
-                ready = self.identifyToPrimaryNode(conn)
+                # Request identification and required informations to be
+                # operational. Might raise ConnectionClosed so that the new
+                # primary can be looked-up again.
+                logging.info('Initializing from master')
+                ask(conn, Packets.AskNodeInformation(), handler=handler)
+                ask(conn, Packets.AskPartitionTable(), handler=handler)
+                ask(conn, Packets.AskLastTransaction(), handler=handler)
+                if self.pt.operational():
+                    break
             except ConnectionClosed:
                 logging.error('Connection to %s lost', self.trying_master_node)
                 self.primary_master_node = None
         logging.info("Connected and ready")
         return conn
 
-    def identifyToPrimaryNode(self, conn):
-        """
-            Request identification and required informations to be operational.
-            Might raise ConnectionClosed so that the new primary can be
-            looked-up again.
-        """
-        logging.info('Initializing from master')
-        ask = self._ask
-        handler = self.primary_bootstrap_handler
-        ask(conn, Packets.AskNodeInformation(), handler=handler)
-        ask(conn, Packets.AskPartitionTable(), handler=handler)
-        self.lastTransaction()
-        return self.pt.operational()
-
     def registerDB(self, db, limit):
         self._db = db
 
     def getDB(self):
         return self._db
 
-    @profiler_decorator
     def new_oid(self):
         """Get a new OID."""
         self._oid_lock_acquire()
         try:
-            if len(self.new_oid_list) == 0:
+            if not self.new_oid_list:
                 # Get new oid list from master node
                 # we manage a list of oid here to prevent
                 # from asking too many time new oid one by one
                 # from master node
                 self._askPrimary(Packets.AskNewOIDs(100))
-                if len(self.new_oid_list) <= 0:
+                if not self.new_oid_list:
                     raise NEOStorageError('new_oid failed')
-            self.last_oid = self.new_oid_list.pop(0)
-            return self.last_oid
+            self.last_oid = oid = self.new_oid_list.pop()
+            return oid
         finally:
             self._oid_lock_release()
 
@@ -372,7 +341,6 @@ class Application(object):
         # return the last OID used, this is inaccurate
         return int(u64(self.last_oid))
 
-    @profiler_decorator
     def load(self, oid, tid=None, before_tid=None):
         """
         Internal method which manage load, loadSerial and loadBefore.
@@ -444,7 +412,6 @@ class Application(object):
         finally:
             self._load_lock_release()
 
-    @profiler_decorator
     def _loadFromStorage(self, oid, at_tid, before_tid):
         packet = Packets.AskObject(oid, at_tid, before_tid)
         for node, conn in self.cp.iterateForObject(oid, readable=True):
@@ -467,7 +434,6 @@ class Application(object):
         # connection error
         raise NEOStorageError('connection failure')
 
-    @profiler_decorator
     def _loadFromCache(self, oid, at_tid=None, before_tid=None):
         """
         Load from local cache, return None if not found.
@@ -478,32 +444,21 @@ class Application(object):
             return result
         return self._cache.load(oid, before_tid)
 
-    @profiler_decorator
     def tpc_begin(self, transaction, tid=None, status=' '):
         """Begin a new transaction."""
-        txn_container = self._txn_container
         # First get a transaction, only one is allowed at a time
-        if txn_container.get(transaction) is not None:
-            # We already begin the same transaction
-            raise StorageTransactionError('Duplicate tpc_begin calls')
-        txn_context = txn_container.new(transaction)
+        txn_context = self._txn_container.new(transaction)
         # use the given TID or request a new one to the master
         answer_ttid = self._askPrimary(Packets.AskBeginTransaction(tid))
         if answer_ttid is None:
             raise NEOStorageError('tpc_begin failed')
         assert tid in (None, answer_ttid), (tid, answer_ttid)
-        txn_context['txn'] = transaction
         txn_context['ttid'] = answer_ttid
 
-    @profiler_decorator
     def store(self, oid, serial, data, version, transaction):
         """Store object."""
-        txn_context = self._txn_container.get(transaction)
-        if txn_context is None:
-            raise StorageTransactionError(self, transaction)
         logging.debug('storing oid %s serial %s', dump(oid), dump(serial))
-        self._store(txn_context, oid, serial, data)
-        return None
+        self._store(self._txn_container.get(transaction), oid, serial, data)
 
     def _store(self, txn_context, oid, serial, data, data_serial=None,
             unlock=False):
@@ -517,8 +472,6 @@ class Application(object):
             checksum = ZERO_HASH
         else:
             assert data_serial is None
-            compression = self.compress
-            compressed_data = data
             size = len(data)
             if self.compress:
                 compressed_data = compress(data)
@@ -527,16 +480,21 @@ class Application(object):
                     compression = 0
                 else:
                     compression = 1
+            else:
+                compression = 0
+                compressed_data = data
             checksum = makeChecksum(compressed_data)
             txn_context['data_size'] += size
-        on_timeout = OnTimeout(self.onStoreTimeout, txn_context, oid)
+        on_timeout = partial(
+            self.onStoreTimeout,
+            txn_context=txn_context,
+            oid=oid,
+        )
         # Store object in tmp cache
         txn_context['data_dict'][oid] = data
         # Store data on each node
         txn_context['object_stored_counter_dict'][oid] = {}
-        object_base_serial_dict = txn_context['object_base_serial_dict']
-        if oid not in object_base_serial_dict:
-            object_base_serial_dict[oid] = serial
+        txn_context['object_base_serial_dict'].setdefault(oid, serial)
         txn_context['object_serial_dict'][oid] = serial
         queue = txn_context['queue']
         involved_nodes = txn_context['involved_nodes']
@@ -566,7 +524,6 @@ class Application(object):
         conn.ask(Packets.AskHasLock(txn_context['ttid'], oid),
                  timeout=5, queue=txn_context['queue'])
 
-    @profiler_decorator
     def _handleConflicts(self, txn_context, tryToResolveConflict):
         result = []
         append = result.append
@@ -637,10 +594,15 @@ class Application(object):
                     # A later serial has already been resolved, skip.
                     resolved_serial_set.update(conflict_serial_set)
                     continue
-                new_data = tryToResolveConflict(oid, conflict_serial,
-                    serial, data)
-                if new_data is not None:
-                    logging.info('Conflict resolution succeed for '
+                try:
+                    new_data = tryToResolveConflict(oid, conflict_serial,
+                        serial, data)
+                except ConflictError:
+                    logging.info('Conflict resolution failed for '
+                        '%r:%r with %r', dump(oid), dump(serial),
+                        dump(conflict_serial))
+                else:
+                    logging.info('Conflict resolution succeeded for '
                         '%r:%r with %r', dump(oid), dump(serial),
                         dump(conflict_serial))
                     # Mark this conflict as resolved
@@ -651,15 +613,10 @@ class Application(object):
                     self._store(txn_context, oid, conflict_serial, new_data)
                     append(oid)
                     continue
-                else:
-                    logging.info('Conflict resolution failed for '
-                        '%r:%r with %r', dump(oid), dump(serial),
-                        dump(conflict_serial))
-            raise ConflictError(oid=oid, serials=(txn_context['ttid'],
+            raise ConflictError(oid=oid, serials=(conflict_serial,
                 serial), data=data)
         return result
 
-    @profiler_decorator
     def waitResponses(self, queue):
         """Wait for all requests to be answered (or their connection to be
         detected as closed)"""
@@ -668,7 +625,6 @@ class Application(object):
         while pending(queue):
             _waitAnyMessage(queue)
 
-    @profiler_decorator
     def waitStoreResponses(self, txn_context, tryToResolveConflict):
         result = []
         append = result.append
@@ -699,54 +655,43 @@ class Application(object):
                 append((oid, ResolvedSerial))
         return result
 
-    @profiler_decorator
     def tpc_vote(self, transaction, tryToResolveConflict):
         """Store current transaction."""
         txn_context = self._txn_container.get(transaction)
-        if txn_context is None or transaction is not txn_context['txn']:
-            raise StorageTransactionError(self, transaction)
-
         result = self.waitStoreResponses(txn_context, tryToResolveConflict)
 
         ttid = txn_context['ttid']
         # Store data on each node
-        txn_stored_counter = 0
         assert not txn_context['data_dict'], txn_context
         packet = Packets.AskStoreTransaction(ttid, str(transaction.user),
             str(transaction.description), dumps(transaction._extension),
             txn_context['cache_dict'])
         add_involved_nodes = txn_context['involved_nodes'].add
         for node, conn in self.cp.iterateForObject(ttid):
-            logging.debug("voting object %s on %s", dump(ttid),
+            logging.debug("voting transaction %s on %s", dump(ttid),
                 dump(conn.getUUID()))
             try:
                 self._askStorage(conn, packet)
             except ConnectionClosed:
                 continue
             add_involved_nodes(node)
-            txn_stored_counter += 1
 
         # check at least one storage node accepted
-        if txn_stored_counter == 0:
-            logging.error('tpc_vote failed')
-            raise NEOStorageError('tpc_vote failed')
-        # Check if master connection is still alive.
-        # This is just here to lower the probability of detecting a problem
-        # in tpc_finish, as we should do our best to detect problem before
-        # tpc_finish.
-        self._getMasterConnection()
+        if txn_context['involved_nodes']:
+            txn_context['voted'] = None
+            # We must not go further if connection to master was lost since
+            # tpc_begin, to lower the probability of failing during tpc_finish.
+            if 'error' in txn_context:
+                raise NEOStorageError(txn_context['error'])
+            return result
+        logging.error('tpc_vote failed')
+        raise NEOStorageError('tpc_vote failed')
 
-        txn_context['txn_voted'] = True
-        return result
-
-    @profiler_decorator
     def tpc_abort(self, transaction):
         """Abort current transaction."""
-        txn_container = self._txn_container
-        txn_context = txn_container.get(transaction)
+        txn_context = self._txn_container.pop(transaction)
         if txn_context is None:
             return
-
         ttid = txn_context['ttid']
         p = Packets.AbortTransaction(ttid)
         getConnForNode = self.cp.getConnForNode
@@ -760,49 +705,41 @@ class Application(object):
             except:
                 logging.exception('Exception in tpc_abort while notifying'
                     'storage node %r of abortion, ignoring.', conn)
-        self._getMasterConnection().notify(p)
-        queue = txn_context['queue']
+        conn = self.master_conn
+        if conn is not None:
+            conn.notify(p)
         # We don't need to flush queue, as it won't be reused by future
         # transactions (deleted on next line & indexed by transaction object
         # instance).
-        self.dispatcher.forget_queue(queue, flush_queue=False)
-        txn_container.delete(transaction)
+        self.dispatcher.forget_queue(txn_context['queue'], flush_queue=False)
 
-    @profiler_decorator
     def tpc_finish(self, transaction, tryToResolveConflict, f=None):
         """Finish current transaction."""
         txn_container = self._txn_container
-        txn_context = txn_container.get(transaction)
-        if txn_context is None:
-            raise StorageTransactionError('tpc_finish called for wrong '
-                'transaction')
-        if not txn_context['txn_voted']:
+        if 'voted' not in txn_container.get(transaction):
             self.tpc_vote(transaction, tryToResolveConflict)
         self._load_lock_acquire()
         try:
             # Call finish on master
+            txn_context = txn_container.pop(transaction)
             cache_dict = txn_context['cache_dict']
             tid = self._askPrimary(Packets.AskFinishTransaction(
                 txn_context['ttid'], cache_dict),
                 cache_dict=cache_dict, callback=f)
-            txn_container.delete(transaction)
+            assert tid
             return tid
         finally:
             self._load_lock_release()
 
     def undo(self, undone_tid, txn, tryToResolveConflict):
         txn_context = self._txn_container.get(txn)
-        if txn_context is None:
-            raise StorageTransactionError(self, undone_tid)
-
         txn_info, txn_ext = self._getTransactionInformation(undone_tid)
         txn_oid_list = txn_info['oids']
 
         # Regroup objects per partition, to ask a minimum set of storage.
         partition_oid_dict = {}
-        pt = self.getPartitionTable()
         for oid in txn_oid_list:
-            partition = pt.getPartition(oid)
+            partition = self.pt.getPartition(oid)
             try:
                 oid_list = partition_oid_dict[partition]
             except KeyError:
@@ -811,10 +748,10 @@ class Application(object):
 
         # Ask storage the undo serial (serial at which object's previous data
         # is)
-        getCellList = pt.getCellList
+        getCellList = self.pt.getCellList
         getCellSortKey = self.cp.getCellSortKey
         getConnForCell = self.cp.getConnForCell
-        queue = self._getThreadQueue()
+        queue = self._thread_container.queue
         ttid = txn_context['ttid']
         undo_object_tid_dict = {}
         snapshot_tid = p64(u64(self.last_tid) + 1)
@@ -897,11 +834,10 @@ class Application(object):
         # First get a list of transactions from all storage nodes.
         # Each storage node will return TIDs only for UP_TO_DATE state and
         # FEEDING state cells
-        pt = self.getPartitionTable()
-        queue = self._getThreadQueue()
+        queue = self._thread_container.queue
         packet = Packets.AskTIDs(first, last, INVALID_PARTITION)
         tid_set = set()
-        for storage_node in pt.getNodeSet(True):
+        for storage_node in self.pt.getNodeSet(True):
             conn = self.cp.getConnForNode(storage_node)
             if conn is None:
                 continue
@@ -985,7 +921,6 @@ class Application(object):
                 self._insertMetadata(txn_info, txn_ext)
             return result
 
-    @profiler_decorator
     def importFrom(self, source, start, stop, tryToResolveConflict,
             preindex=None):
         if preindex is None:
@@ -1015,16 +950,6 @@ class Application(object):
         self._askPrimary(Packets.AskLastTransaction())
         return self.last_tid
 
-    def abortVersion(self, src, transaction):
-        if self._txn_container.get(transaction) is None:
-            raise StorageTransactionError(self, transaction)
-        return '', []
-
-    def commitVersion(self, src, dest, transaction):
-        if self._txn_container.get(transaction) is None:
-            raise StorageTransactionError(self, transaction)
-        return '', []
-
     def __del__(self):
         """Clear all connection."""
         # Due to bug in ZODB, close is not always called when shutting
@@ -1058,10 +983,8 @@ class Application(object):
         return self.load(oid)[1]
 
     def checkCurrentSerialInTransaction(self, oid, serial, transaction):
-        txn_context = self._txn_container.get(transaction)
-        if txn_context is None:
-              raise StorageTransactionError(self, transaction)
-        self._checkCurrentSerialInTransaction(txn_context, oid, serial)
+        self._checkCurrentSerialInTransaction(
+            self._txn_container.get(transaction), oid, serial)
 
     def _checkCurrentSerialInTransaction(self, txn_context, oid, serial):
         ttid = txn_context['ttid']
