@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2006-2011  Nexedi SA
+# Copyright (C) 2006-2014  Nexedi SA
 
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -59,9 +59,8 @@ class ClientServiceHandler(MasterHandler):
 
         # Collect partitions related to this transaction.
         getPartition = app.pt.getPartition
-        partition_set = set()
+        partition_set = set(getPartition(oid) for oid in oid_list)
         partition_set.add(getPartition(ttid))
-        partition_set.update((getPartition(oid) for oid in oid_list))
 
         # Collect the UUIDs of nodes related to this transaction.
         uuid_set = set()
@@ -76,16 +75,21 @@ class ClientServiceHandler(MasterHandler):
             raise ProtocolError('No storage node ready for transaction')
 
         identified_node_list = app.nm.getIdentifiedList(pool_set=uuid_set)
-        usable_uuid_set = set((x.getUUID() for x in identified_node_list))
-        partitions = app.pt.getPartitions()
-        peer_id = conn.getPeerId()
-        tid = app.tm.prepare(ttid, partitions, oid_list, usable_uuid_set,
-            peer_id)
 
         # Request locking data.
         # build a new set as we may not send the message to all nodes as some
         # might be not reachable at that time
-        p = Packets.AskLockInformation(ttid, tid, oid_list)
+        p = Packets.AskLockInformation(
+            ttid,
+            app.tm.prepare(
+                ttid,
+                app.pt.getPartitions(),
+                oid_list,
+                set(x.getUUID() for x in identified_node_list),
+                conn.getPeerId(),
+            ),
+            oid_list,
+        )
         for node in identified_node_list:
             node.ask(p, timeout=60)
 
