@@ -1,4 +1,4 @@
-#! /usr/bin/env python2.4
+#!/usr/bin/env python
 #
 # neomaster - run a master node of NEO
 #
@@ -15,11 +15,9 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 from optparse import OptionParser
-import logging
 import time
 import os
 
