@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2009-2012  Nexedi SA
+# Copyright (C) 2009-2014  Nexedi SA
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -44,11 +44,6 @@ def _getMasterConnection(self):
         self.master_conn = Mock()
     return self.master_conn
 
-def getPartitionTable(self):
-    if self.pt is None:
-        self.master_conn = _getMasterConnection(self)
-    return self.pt
-
 def _ask(self, conn, packet, handler=None, **kw):
     self.setHandlerData(None)
     conn.ask(packet, **kw)
@@ -62,7 +57,7 @@ def resolving_tryToResolveConflict(oid, conflict_serial, serial, data):
     return data
 
 def failing_tryToResolveConflict(oid, conflict_serial, serial, data):
-    return None
+    raise ConflictError
 
 class ClientApplicationTests(NeoUnitTestBase):
 
@@ -71,10 +66,8 @@ class ClientApplicationTests(NeoUnitTestBase):
         # apply monkey patches
         self._getMasterConnection = Application._getMasterConnection
         self._ask = Application._ask
-        self.getPartitionTable = Application.getPartitionTable
         Application._getMasterConnection = _getMasterConnection
         Application._ask = _ask
-        Application.getPartitionTable = getPartitionTable
         self._to_stop_list = []
 
     def _tearDown(self, success):
@@ -82,9 +75,8 @@ class ClientApplicationTests(NeoUnitTestBase):
         for app in self._to_stop_list:
             app.close()
         # restore environnement
-        Application._getMasterConnection = self._getMasterConnection
         Application._ask = self._ask
-        Application.getPartitionTable = self.getPartitionTable
+        Application._getMasterConnection = self._getMasterConnection
         NeoUnitTestBase._tearDown(self, success)
 
     # some helpers
@@ -245,7 +237,7 @@ class ClientApplicationTests(NeoUnitTestBase):
         tid = self.makeTID()
         txn = Mock()
         # first, tid is supplied
-        self.assertTrue(app._txn_container.get(txn) is None)
+        self.assertRaises(StorageTransactionError, app._txn_container.get, txn)
         packet = Packets.AnswerBeginTransaction(tid=tid)
         packet.setId(0)
         app.master_conn = Mock({
@@ -419,7 +411,7 @@ class ClientApplicationTests(NeoUnitTestBase):
         self.checkNotifyPacket(conn1, Packets.AbortTransaction)
         self.checkNotifyPacket(conn2, Packets.AbortTransaction)
         self.checkNotifyPacket(app.master_conn, Packets.AbortTransaction)
-        self.assertEqual(app._txn_container.get(txn), None)
+        self.assertRaises(StorageTransactionError, app._txn_container.get, txn)
 
     def test_tpc_abort3(self):
         """ check that abort is sent to all nodes involved in the transaction """
@@ -499,11 +491,11 @@ class ClientApplicationTests(NeoUnitTestBase):
             'getAddress': ('127.0.0.1', 10010),
             'fakeReceived': packet,
         })
-        txn_context['txn_voted'] = True
+        txn_context['voted'] = None
         app.tpc_finish(txn, None)
         self.checkAskFinishTransaction(app.master_conn)
         #self.checkDispatcherRegisterCalled(app, app.master_conn)
-        self.assertEqual(app._txn_container.get(txn), None)
+        self.assertRaises(StorageTransactionError, app._txn_container.get, txn)
 
     def test_undo1(self):
         # invalid transaction
@@ -843,16 +835,16 @@ class ClientApplicationTests(NeoUnitTestBase):
         """ Thread context properties must not be visible accross instances
             while remaining in the same thread """
         app1 = self.getApp()
-        app1_local = app1._thread_container.get()
+        app1_local = app1._thread_container
         app2 = self.getApp()
-        app2_local = app2._thread_container.get()
+        app2_local = app2._thread_container
         property_id = 'thread_context_test'
         value = 'value'
-        self.assertRaises(KeyError, app1_local.__getitem__, property_id)
-        self.assertRaises(KeyError, app2_local.__getitem__, property_id)
-        app1_local[property_id] = value
-        self.assertEqual(app1_local[property_id], value)
-        self.assertRaises(KeyError, app2_local.__getitem__, property_id)
+        self.assertFalse(hasattr(app1_local, property_id))
+        self.assertFalse(hasattr(app2_local, property_id))
+        setattr(app1_local, property_id, value)
+        self.assertEqual(getattr(app1_local, property_id), value)
+        self.assertFalse(hasattr(app2_local, property_id))
 
     def test_pack(self):
         app = self.getApp()
