@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2011-2012  Nexedi SA
+# Copyright (C) 2011-2014  Nexedi SA
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -23,11 +23,11 @@ from persistent import Persistent
 from ZODB import POSException
 from neo.storage.transactions import TransactionManager, \
     DelayedError, ConflictError
-from neo.lib.connection import MTClientConnection
+from neo.lib.connection import ConnectionClosed, MTClientConnection
 from neo.lib.protocol import CellStates, ClusterStates, NodeStates, Packets, \
     ZERO_TID
 from . import ClientApplication, NEOCluster, NEOThreadedTest, Patch
-from neo.lib.util import makeChecksum
+from neo.lib.util import add64, makeChecksum
 from neo.client.pool import CELL_CONNECTED, CELL_GOOD
 
 class PCounter(Persistent):
@@ -586,6 +586,7 @@ class Test(NEOThreadedTest):
                     l1.release()
                     l2.acquire()
             x2._p_deactivate()
+            # Remove last version of x from cache
             cache._remove(cache._oid_dict[x2._p_oid].pop())
             p = Patch(cluster.client, _loadFromStorage=_loadFromStorage)
             try:
@@ -593,13 +594,13 @@ class Test(NEOThreadedTest):
                 l1.acquire()
                 # At this point, x could not be found the cache and the result
                 # from the storage (which is <value=1, next_tid=None>) is about
-                # to processed.
+                # to be processed.
                 # Now modify x to receive an invalidation for it.
                 cluster.client.setPoll(0)
                 client.setPoll(1)
                 txn = transaction.Transaction()
                 client.tpc_begin(txn)
-                client.store(x2._p_oid, tid, x, '', txn)
+                client.store(x2._p_oid, tid, x, '', txn) # value=0
                 tid = client.tpc_finish(txn, None)
                 client.setPoll(0)
                 cluster.client.setPoll(1)
@@ -613,6 +614,10 @@ class Test(NEOThreadedTest):
             self.assertEqual(x2.value, 1)
             self.assertEqual(x1.value, 0)
 
+            # l1 is acquired and l2 is released
+            # Change x again from 0 to 1, while the checking connection c1
+            # is suspended at the beginning of the transaction t1,
+            # between Storage.sync() and flush of invalidations.
             def _flush_invalidations(orig):
                 l1.release()
                 l2.acquire()
@@ -636,11 +641,87 @@ class Test(NEOThreadedTest):
                 del p
                 l2.release()
             t.join()
+            # A transaction really begins when it acquires the lock to flush
+            # invalidations. The previous lastTransaction() only does a ping
+            # to make sure we have a recent enough view of the DB.
             self.assertEqual(x1.value, 1)
 
         finally:
             cluster.stop()
 
+    def testClientReconnection(self):
+        cluster = NEOCluster()
+        try:
+            cluster.start()
+            t1, c1 = cluster.getTransaction()
+            c1.root()['x'] = x1 = PCounter()
+            c1.root()['y'] = y = PCounter()
+            y.value = 1
+            t1.commit()
+            x = c1._storage.load(x1._p_oid)[0]
+            y = c1._storage.load(y._p_oid)[0]
+
+            # close connections to master & storage
+            cluster.client.setPoll(0)
+            c, = cluster.master.nm.getClientList()
+            c.getConnection().close()
+            c, = cluster.storage.nm.getClientList()
+            c.getConnection().close()
+            cluster.tic(force=1)
+
+            # modify x with another client
+            client = ClientApplication(name=cluster.name,
+                                       master_nodes=cluster.master_nodes)
+            client.setPoll(1)
+            txn = transaction.Transaction()
+            client.tpc_begin(txn)
+            client.store(x1._p_oid, x1._p_serial, y, '', txn)
+            tid = client.tpc_finish(txn, None)
+            client.close()
+            client.setPoll(0)
+            cluster.client.setPoll(1)
+
+            t1.begin()
+            self.assertEqual(x1._p_changed ,None)
+            self.assertEqual(x1.value, 1)
+        finally:
+            cluster.stop()
+
+    def testInvalidTTID(self):
+        cluster = NEOCluster()
+        try:
+            cluster.start()
+            client = cluster.client
+            client.setPoll(1)
+            txn = transaction.Transaction()
+            client.tpc_begin(txn)
+            txn_context = client._txn_container.get(txn)
+            txn_context['ttid'] = add64(txn_context['ttid'], 1)
+            self.assertRaises(POSException.StorageError,
+                              client.tpc_finish, txn, None)
+        finally:
+            cluster.stop()
+
+    def testStorageFailureDuringTpcFinish(self):
+        def answerTransactionFinished(conn, packet):
+            if isinstance(packet, Packets.AnswerTransactionFinished):
+                c, = cluster.storage.getConnectionList(cluster.master)
+                c.abort()
+        cluster = NEOCluster()
+        try:
+            cluster.start()
+            t, c = cluster.getTransaction()
+            c.root()['x'] = PCounter()
+            with cluster.master.filterConnection(cluster.client) as m2c:
+                m2c.add(answerTransactionFinished)
+                # XXX: This is an expected failure. A ttid column was added to
+                #      'trans' table to permit recovery, by checking that the
+                #      transaction was really committed.
+                self.assertRaises(ConnectionClosed, t.commit)
+            t.begin()
+            c.root()['x']
+        finally:
+            cluster.stop()
 
 if __name__ == "__main__":
     unittest.main()
